@@ -39,6 +39,8 @@
 #include "types-internal.h"
 #include "debug.h"
 #include "util/rbtree.h"
+#include "util-internal.h"
+#include "platform.h"
 #include "server.h"
 
 #define DNS_REQUEST_SZ          4096
@@ -135,11 +137,7 @@ static void tcp_connection_destroy(tcp_connection *conn)
 		loop->vmt->clear(loop, &conn->event);
 
 	if (conn->fd >= 0)
-#ifdef USE_WINSOCK
-		(void) closesocket(conn->fd);
-#else
-		(void) close(conn->fd);
-#endif
+		(void) _getdns_closesocket(conn->fd);
 	GETDNS_FREE(*mf, conn->read_buf);
 
 	for (cur = conn->to_write; cur; cur = next) {
@@ -189,9 +187,16 @@ static void tcp_write_cb(void *userarg)
 	}
 	to_write = conn->to_write;
 	if (conn->fd == -1 || 
-	    (written = send(conn->fd, &to_write->write_buf[to_write->written],
+	    (written = send(conn->fd,
+	    (const void *)&to_write->write_buf[to_write->written],
 	    to_write->write_buf_len - to_write->written, 0)) == -1) {
 
+		if (_getdns_socketerror_wants_retry())
+			return;
+
+		DEBUG_SERVER("I/O error from send(): %s\n",
+	        	     _getdns_errnostr());
+
 		/* IO error, close connection */
 		conn->event.read_cb = conn->event.write_cb =
 		    conn->event.timeout_cb = NULL;
@@ -282,14 +287,11 @@ getdns_reply(
 
 		if (conn->l->fd >= 0 && sendto(conn->l->fd, (void *)buf, len, 0,
 		    (struct sockaddr *)&conn->remote_in, conn->addrlen) == -1) {
-			/* IO error, cleanup this listener */
-			loop->vmt->clear(loop, &conn->l->event);
-#ifdef USE_WINSOCK
-			closesocket(conn->l->fd);
-#else
-			close(conn->l->fd);
-#endif
-			conn->l->fd = -1;
+			/* TODO: handle _getdns_socketerror_wants_retry() */
+
+			/* IO error, never cleanup a listener because of I/O error */
+			DEBUG_SERVER("I/O error from sendto(): %s\n",
+				     _getdns_errnostr());
 		}
 		/* Unlink this connection */
 		(void) _getdns_rbtree_delete(
@@ -367,11 +369,15 @@ static void tcp_read_cb(void *userarg)
 	(void) loop->vmt->schedule(loop, conn->fd,
 	    DOWNSTREAM_IDLE_TIMEOUT, &conn->event);
 
-	if ((bytes_read = recv(conn->fd, conn->read_pos, conn->to_read, 0)) < 0) {
-		if (errno == EAGAIN || errno == EWOULDBLOCK)
+	if ((bytes_read = recv(conn->fd,
+	    (void *)conn->read_pos, conn->to_read, 0)) < 0) {
+		if (_getdns_socketerror_wants_retry())
 			return; /* Come back to do the read later */
 
 		/* IO error, close connection */
+		DEBUG_SERVER("I/O error from recv(): %s\n",
+	        	     _getdns_errnostr());
+
 		tcp_connection_destroy(conn);
 		return;
 	}
@@ -479,14 +485,19 @@ static void tcp_accept_cb(void *userarg)
 	conn->super.addrlen = sizeof(conn->super.remote_in);
 	if ((conn->fd = accept(l->fd, (struct sockaddr *)
 	    &conn->super.remote_in, &conn->super.addrlen)) == -1) {
-		/* IO error, cleanup this listener */
-		loop->vmt->clear(loop, &l->event);
-#ifdef USE_WINSOCK
-		closesocket(l->fd);
-#else
-		close(l->fd);
-#endif
-		l->fd = -1;
+
+		if (_getdns_socketerror_wants_retry() ||
+		    _getdns_socketerror() == _getdns_ECONNRESET)
+			; /* pass */
+
+		else if (_getdns_resource_depletion())
+			; /* TODO: Stop listening for a little while? */
+
+		else
+			DEBUG_SERVER("I/O error during accept: %s\n",
+			             _getdns_errnostr());
+
+		/* Never cleanup a listener because of I/O errors! */
 		GETDNS_FREE(*mf, conn);
 		return;
 	}
@@ -553,14 +564,17 @@ static void udp_read_cb(void *userarg)
 	conn->addrlen = sizeof(conn->remote_in);
 	if ((len = recvfrom(l->fd, (void *)buf, sizeof(buf), 0,
 	    (struct sockaddr *)&conn->remote_in, &conn->addrlen)) == -1) {
-		/* IO error, cleanup this listener. */
-		loop->vmt->clear(loop, &l->event);
-#ifdef USE_WINSOCK
-		closesocket(l->fd);
-#else
-		close(l->fd);
-#endif
-		l->fd = -1;
+		if ( _getdns_socketerror_wants_retry() &&
+		    _getdns_socketerror() != _getdns_ECONNRESET) {
+			/*
+			 * WINSOCK gives ECONNRESET on ICMP Port Unreachable
+			 * being received. Ignore it.
+			 */
+			DEBUG_SERVER("I/O error from recvfrom: %s\n",
+		        	     _getdns_errnostr());
+
+		}
+		/* Never cleanup a listener because of an I/O error! */
 
 #if 0 && defined(SERVER_DEBUG) && SERVER_DEBUG
 	} else {
@@ -708,11 +722,7 @@ static void remove_listeners(listen_set *set)
 			continue;
 
 		loop->vmt->clear(loop, &l->event);
-#ifdef USE_WINSOCK
-		closesocket(l->fd);
-#else
-		close(l->fd);
-#endif
+		_getdns_closesocket(l->fd);
 		l->fd = -1;
 
 		if (l->transport != GETDNS_TRANSPORT_TCP)
