@@ -35,12 +35,12 @@
 
 const char *getdns_get_version(void)
 {
-	return "1.2.1-rc1";
+	return "1.3.0";
 }
 
 uint32_t getdns_get_version_number(void)
 {
-	return 0x010200C1;
+	return 0x01030000;
 }
 
 const char *getdns_get_api_version(void)
